local KHMRaidFrames = LibStub("AceAddon-3.0"):GetAddon("KHMRaidFrames")
local SharedMedia = LibStub:GetLibrary("LibSharedMedia-3.0")

local _G, tinsert, CompactRaidFrameContainer = _G, tinsert, CompactRaidFrameContainer
KHMRaidFrames.NATIVE_UNIT_FRAME_HEIGHT = 36
KHMRaidFrames.NATIVE_UNIT_FRAME_WIDTH = 72

KHMRaidFrames.defuffsColors = {
    magic = {0.2, 0.6, 1.0, 1},
    curse = {0.6, 0.0, 1.0, 1},
    disease = {0.6, 0.4, 0.0, 1},
    poison = {0.0, 0.6, 0.0, 1},
    physical = {1, 1, 1, 1}
}

function KHMRaidFrames:Defaults()
    local defaults_settings = {profile = {party = {}, raid = {}, glows = {}}}

    local commons = {
            frames = {
                hideGroupTitles = false,
                texture = SharedMedia.DefaultMedia.statusbar or "Blizzard Raid Bar",
                clickThrough = false,
                enhancedAbsorbs = false,
                showPartySolo = false,
                tracking = {},
                trackingStr = "",
            },
            dispelDebuffFrames = {
                num = 3,
                numInRow = 3,
                rowsGrowDirection = "TOP",
                anchorPoint = "TOPRIGHT",
                growDirection = "LEFT",
                size = 12,
                xOffset = 0,
                yOffset = 0,
                exclude = {},
                excludeStr = "",
                alpha = 1.0,
            },
            debuffFrames = {
                num = 3,
                numInRow = 3,
                rowsGrowDirection = "TOP",
                anchorPoint = "BOTTOMLEFT",
                growDirection = "RIGHT",
                size = 11,
                xOffset = 0,
                yOffset = 0,
                exclude = {},
                excludeStr = "",
                bigDebuffSize = 11 + 9,
                showBigDebuffs = true,
                smartAnchoring = true,
                alpha = 1.0,
            },
            buffFrames = {
                num = 3,
                numInRow = 3,
                rowsGrowDirection = "TOP",
                anchorPoint = "BOTTOMRIGHT",
                growDirection = "LEFT",
                size = 11,
                xOffset = 0,
                yOffset = 0,
                exclude = {},
                excludeStr = "",
                alpha = 1.0,
            },
            raidIcon = {
                enabled = false,
                size = 15,
                xOffset = 0,
                yOffset = 0,
                anchorPoint = "TOP",
                alpha = 1.0,
            },
            nameAndIcons = {
                name = {
                    font = SharedMedia.DefaultMedia.font or "Friz Quadrata TT",
                    size = 6,
                    flag = "None",
                    hJustify = "LEFT",
                    xOffset = 0,
                    yOffset = -1,
                    showServer = true,
                    classColoredNames = false,
                    enabled = false,
                },
                statusText = {
                    font = SharedMedia.DefaultMedia.font or "Friz Quadrata TT",
                    size = 12,
                    flag = "None",
                    hJustify = "CENTER",
                    xOffset = 0,
                    yOffset = 0,
                    enabled = false,
                    abbreviateNumbers = "None",
                    notShowStatuses = false,
                },
                roleIcon = {
                    size = 12,
                    xOffset = 0,
                    yOffset = 0,
                    healer = "",
                    damager = "",
                    tank = "",
                    vehicle = "",
                    toggle = false,
                    enabled = false,
                    colors = {
                        healer = {1, 1, 1, 1},
                        damager = {1, 1, 1, 1},
                        tank = {1, 1, 1, 1},
                        vehicle = {1, 1, 1, 1},
                    },
                },
                readyCheckIcon  = {
                    size = 15 ,
                    xOffset = 0,
                    yOffset = 0,
                    ready = "",
                    notready = "",
                    waiting = "",
                    toggle = false,
                    enabled = false,
                    colors = {
                        ready = {1, 1, 1, 1},
                        notready = {1, 1, 1, 1},
                        waiting = {1, 1, 1, 1},
                    },
                },
                centerStatusIcon = {
                    size = 22,
                    xOffset = 0,
                    yOffset = 0,
                    inOtherGroup = "",
                    hasIncomingResurrection = "",
                    hasIncomingSummonPending = "",
                    hasIncomingSummonAccepted = "",
                    hasIncomingSummonDeclined = "",
                    inOtherPhase = "",
                    toggle = false,
                    enabled = false,
                    colors = {
                        inOtherGroup = {1, 1, 1, 1},
                        hasIncomingResurrection = {1, 1, 1, 1},
                        hasIncomingSummonPending = {1, 1, 1, 1},
                        hasIncomingSummonAccepted = {1, 1, 1, 1},
                        hasIncomingSummonDeclined = {1, 1, 1, 1},
                        inOtherPhase = {1, 1, 1, 1},
                    },
                },
                leaderIcon = {
                    size = 10 ,
                    xOffset = 0,
                    yOffset = 0,
                    anchorPoint = "TOPRIGHT",
                    icon = "",
                    enabled = false,
                    alpha = 1.0,
                    colors = {
                        icon = {1, 1, 1, 1},
                    },
                },
            },
    }
    defaults_settings.profile.party = commons
    defaults_settings.profile.raid = commons

    defaults_settings.profile.glows = {
        auraGlow = {
            buffFrames = {
                type = "pixel",
                options = self.GetGlowOptions(),
                tracking = {},
                trackingStr = "",
                enabled = false,
                useDefaultsColors = true,
            },
            debuffFrames = {
                type = "pixel",
                options = self.GetGlowOptions(),
                tracking = {},
                trackingStr = "",
                enabled = false,
                useDefaultsColors = true,
            },
            defaultColors = self.defuffsColors,
        },
        frameGlow = {
            buffFrames = {
                type = "pixel",
                options = self.GetGlowOptions(),
                tracking = {},
                trackingStr = "",
                enabled = false,
                useDefaultsColors = true,
            },
            debuffFrames = {
                type = "pixel",
                options = self.GetGlowOptions(),
                tracking = {},
                trackingStr = "",
                enabled = false,
                useDefaultsColors = true,
            },
            defaultColors = self.defuffsColors,
        },
        glowBlockList = {
            tracking = {},
            trackingStr = "",
        },
    }

    defaults_settings.profile.Masque = false

    return defaults_settings
end

function KHMRaidFrames:RestoreDefaults(groupType, frameType, subType)
    if InCombatLockdown() then
        print("Can not refresh settings while in combat")
        return
    end

    local defaults_settings = subType and self:Defaults()["profile"][groupType][frameType][subType]

    for k, v in pairs(defaults_settings) do
        self.db.profile[groupType][frameType][k] = v
    end

    self:SafeRefresh(groupType)
end

function KHMRaidFrames:CopySettings(dbFrom, dbTo)
    if InCombatLockdown() then
        print("Can not refresh settings while in combat")
        return
    end

    for k, v in pairs(dbFrom) do
        if dbTo[k] ~= nil then
            dbTo[k] = v
        end
    end

    self:SafeRefresh(groupType)
end

function KHMRaidFrames:CUFDefaults(groupType)
    local deferred
    local isInCombatLockDown = InCombatLockdown()

    for group in self.IterateCompactGroups(groupType) do
        if self.processedFrames[group] == nil then
            deferred = self:DefaultGroupSetUp(group, groupType, isInCombatLockDown)

            if deferred == false then
                self.processedFrames[group] = true
            end
        end
    end

    for frame in self.IterateCompactFrames(groupType) do
        if self.processedFrames[frame] == nil then
            deferred = self:DefaultFrameSetUp(frame, groupType, isInCombatLockDown)

            if deferred == false then
                self.processedFrames[frame] = true
            end
        end
    end

    self:SetUpSoloFrame()
end

function KHMRaidFrames:DefaultGroupSetUp(frame, groupType, isInCombatLockDown)
    local db = self.db.profile[groupType]

    if db.frames.hideGroupTitles then
        frame.title:Hide()
    else
        frame.title:Show()
    end

end

function KHMRaidFrames:DefaultFrameSetUp(frame, groupType, isInCombatLockDown)
    local db = self.db.profile[groupType]
    local deferred = false

    if not isInCombatLockDown then
        self:AddSubFrames(frame, groupType)
    else
        deferred = true
    end

    self:SetUpSubFramesPositionsAndSize(frame, frame.buffFrames, db.buffFrames, groupType, "buffFrames")
    self:SetUpSubFramesPositionsAndSize(frame, frame.debuffFrames, db.debuffFrames, groupType, "debuffFrames")

    if db.showBigDebuffs and db.smartAnchoring then
        self:SmartAnchoring(frame, frame.debuffFrames, db.debuffFrames)
    end

    self:SetUpSubFramesPositionsAndSize(frame, frame.dispelDebuffFrames, db.dispelDebuffFrames, groupType, "dispelDebuffFrames")

    self:SetUpRaidIcon(frame, groupType)

    if self.db.profile[groupType].nameAndIcons.name.enabled then
        self:SetUpName(frame, groupType)
    end

    if self.db.profile[groupType].nameAndIcons.statusText.enabled then
        self:SetUpStatusText(frame, groupType)
    end

    if self.db.profile[groupType].nameAndIcons.roleIcon.enabled then
        self:SetUpRoleIcon(frame, groupType)
    end

    if self.db.profile[groupType].nameAndIcons.readyCheckIcon.enabled then
        self:SetUpReadyCheckIcon(frame, groupType)
    end

    if self.db.profile[groupType].nameAndIcons.centerStatusIcon.enabled then
        self:SetUpCenterStatusIcon(frame, groupType)
    end

    if self.db.profile[groupType].nameAndIcons.leaderIcon.enabled then
        self.SetUpLeaderIcon(frame, groupType)
    end

    local texture = self.textures[db.frames.texture] or self.textures[self:Defaults().profile[groupType].frames.texture]
    frame.healthBar:SetStatusBarTexture(texture, "BORDER")

    return deferred
end